## Bias Correction Master Script
## Created by Bryan M. Williams on January 29, 2014

import time as t
import sys as s
import os, shutil

###

def configcreator(gsi_dir, run_name): ## Function to create a new config file
        output = open('bc_new_config', 'w')

        for line in open('bc_config'):
                linelist = line.split(' ')
                try:
                        if linelist[1] == 'GSI_DIR':
                                output.write('setenv GSI_DIR ' + str(gsi_dir) + '\n')
                        elif linelist[1] == 'RUN_NAME':
                                output.write('setenv RUN_NAME ' + str(run_name) + '\n')
                        else:
                                output.write(line + '\n')
                except:
                        continue

        output.close()

        try:
                os.remove('./bc_config')
                os.rename('./bc_new_config', './bc_config')
        except:
                os.rename('./bc_new_config', './bc_config')

###

def datestring(date): ## Function to convert date list into a string
        stryear = str(date[0])

        if len(str(date[1])) == 1: ## add 0 if month is earlier than October
                strmon = '0' + str(date[1])
        else:
                strmon = str(date[1])

        if len(str(date[2])) == 1: ## add 0 if day is less than 10
                strday = '0' + str(date[2])
        else:
                strday = str(date[2])

        if len(str(date[3])) == 1: ## Add 0 if hour is less than 10
                strhour = '0' + str(date[3])
        else:
                strhour = str(date[3])

        strdate = stryear + strmon + strday + strhour # Create string and return.
        return strdate

###

def findlastday(year, mon): ## Function to find the last day of a month.
        mon30 = [4, 6, 9, 11]
        lastday = 0

        for M in mon30:
                if int(mon) == M:
                        lastday = 30 ## 30-day month (April, June, September, November)
                        break

        if lastday == 0 and mon != 2:
                lastday = 31 ## 31-day month

        if int(mon) == 2:
                if int(year) % 4 == 0:
                        lastday = 29 ## February during a leap year
                else:
                        lastday = 28 ## February not during a leap year
        return lastday

###

def datelistgen(range, step): ## Function to generate a list of dates.
        L = []
        L = range.split('-')
        startdate = L[0]
        enddate = L[1]
#       print 'Start Date: ' + str(startdate) + '; End date: ' + str(enddate)

        ## Parse the dates into its components: year, month, day, hour. Create a list with these components.

        styear = startdate[0:4]
        stmon = startdate[4:6]
        stday = startdate[6:8]
        sthour = startdate[8:]

        enyear = enddate[0:4]
        enmon = enddate[4:6]
        enday = enddate[6:8]
        enhour = enddate[8:]

        st_year = int(styear)
        st_mon = int(stmon)
        st_day = int(stday)
        st_hour = int(sthour)

        en_year = int(enyear)
        en_mon = int(enmon)
        en_day = int(enday)
        en_hour = int(enhour)

        st_date = [st_year, st_mon, st_day, st_hour]
        en_date = [en_year, en_mon, en_day, en_hour]

        ## Generate list of datelist days.

        datelist = []

        datelist.append(startdate)
        curdate = startdate
        in_date = st_date

        in_year = in_date[0]
        in_mon = in_date[1]
        in_day = in_date[2]
        in_hour = in_date[3]

        while curdate != enddate:

                lastday = findlastday(in_year, in_mon)
                in_hour = in_hour + int(step)

                if in_hour >= 24:
                        in_hour = in_hour - 24
                        in_day = in_day + 1

                if in_day > lastday:
                        in_day = in_day - lastday
                        in_mon = in_mon + 1

                in_date[1] = in_mon
                in_date[2] = in_day
                in_date[3] = in_hour

                datelist.append(datestring(in_date))
                curdate = datestring(in_date)

        return datelist ## Return the date list to the main script.

###

def datedecoder(path, date): ## Function to insert dates into map config directory to insert into the bash script's config.
#       print path

        year = date[0:4]
        mon = date[4:6]
        day = date[6:8]
        hour = date[8:]

        ## First, replace 'YYYY' with the year.

        A = path.find('YYYY')
        while A != -1:
                B = A + 4
                path = path[:A] + str(year) + path[B:]
                A = path.find('YYYY')

        ## Next, add the month (MM).

        A = path.find('MM')
        while A != -1:
                B = A + 2
                path = path[:A] + str(mon) + path[B:]
                A = path.find('MM')

        ## Next, add the day (DD).

        A = path.find('DD')
        while A != -1:
                B = A + 2
                path = path[:A] + str(day) + path[B:]
                A = path.find('DD')

        ## Finally, add the hour (HH).

        A = path.find('HH')
        while A != -1:
                B = A + 2
                path = path[:A] + str(hour) + path[B:]
                A = path.find('HH')


        return path

###

## Open the directories file and create a dictionary for directories.

dirdict = {}

for line in open('directories'):
#       print line
        L = []
        L = line.split(' ')
#       print L
        name = L[0]
        path = L[1]
        path = path.strip('\n\r')
        dirdict[name] = path

experiment = dirdict.keys()

## Open the range config file and create a dictionary for ranges.

rangedict = {}

for line in open('ranges'):
#       print line
        L = []
        L = line.split(' ')
#       print L
        name = L[0]
        rangbegin = L[1]
        rangend = L[2]
        rangend = rangend.strip('\n\r')
        rangedict[name] = rangbegin + '-' + rangend

## Assign variables to command line arguments

try:
        fstep = s.argv[1]
except:
        print 'Script usage: python bias_master.py step'
        quit()

first = True

## Create bias plots.

for exp in experiment:

	if first:
		print 'Starting first run...'
		t.sleep(1)
		first = False
	else:
		print 'Starting next run...'
		t.sleep(1)

	gsi_direc = dirdict[exp]

	print 'Writing to config...'

	configcreator(gsi_direc, exp)
	
	t.sleep(1)

	print 'Running bias_correction5.sh for run ' + str(exp) + '...'
	t.sleep(1)
	
	os.system('./bias_correction5.sh' + ' ' + str(rangbegin) + ' ' + str(rangend) + ' ' + str(fstep))

print 'The Bias Master is finished and will now terminate.'
quit()
